#include "drawwidget.h"
#include <QMenu>
#include <QMouseEvent>
#include <QPainter>
#include <QPen>
#include <QtDebug>


DrawWidget::DrawWidget(QWidget *parent) : QWidget(parent) {
	connect(&_markerTimer, &QTimer::timeout, this, &DrawWidget::onMarkerTimer);
	_markerTimer.start(100);
}


void DrawWidget::mouseMoveEvent(QMouseEvent *event) {
	if (event->buttons() != Qt::NoButton) {
		extractMouseInfo(event);
	}
}


void DrawWidget::mousePressEvent(QMouseEvent *event) {
	extractMouseInfo(event);
	_markers.emplace_back(event->x(), event->y(), _counter + 50);
	update();
}


void DrawWidget::mouseReleaseEvent(QMouseEvent *event) {
	extractMouseInfo(event);
}


void DrawWidget::paintEvent(QPaintEvent *event) {
	Q_UNUSED(event);
	QPainter painter(this);
	painter.setRenderHint(QPainter::Antialiasing, true);
	
	painter.setPen(QPen(Qt::black, 1.5));
	
	for (const Marker &marker : _markers) {
		int x = std::get<0>(marker);
		int y = std::get<1>(marker);
		
		painter.drawLine(x - 5, y - 5, x + 5, y + 5);
		painter.drawLine(x - 5, y + 5, x + 5, y - 5);
	}
}


void DrawWidget::onMarkerTimer() {
	++_counter;
	
	while (_markers.size() > 0 and std::get<2>(_markers.front()) < _counter) {
		_markers.pop_front();
		update();
	}
}


void DrawWidget::extractMouseInfo(QMouseEvent *event) {
	QPoint localPos = event->pos();
	QPointF localPosF = event->localPos();
	QPointF screenPos = event->screenPos();
	QPoint globalPos = event->globalPos();
	QPointF windowPos = event->windowPos();
	
	Q_ASSERT(globalPos.x() == event->globalX());
	Q_ASSERT(globalPos.y() == event->globalY());
	Q_ASSERT(localPos.x() == event->x());
	Q_ASSERT(localPos.y() == event->y());
	
	emit mouseInfo(localPos, localPosF, globalPos, screenPos, windowPos);
}
