#ifndef DRAWWIDGET_H
#define DRAWWIDGET_H

#include <QPoint>
#include <QPointF>
#include <QObject>
#include <QTimer>
#include <QWidget>
#include <deque>
#include <tuple>


class DrawWidget : public QWidget {
	Q_OBJECT
public:
	explicit DrawWidget(QWidget *parent = nullptr);
	
signals:
	void mouseInfo(const QPoint &pos, const QPointF &posF, const QPoint &globalPos,
	               const QPointF &screenPos, const QPointF &windowPos);
	
protected:
	virtual void mouseMoveEvent(QMouseEvent *event) override;
	virtual void mousePressEvent(QMouseEvent *event) override;
	virtual void mouseReleaseEvent(QMouseEvent *event) override;
	
	virtual void paintEvent(QPaintEvent *event) override;

private slots:
	void onMarkerTimer();
	
private:
	void extractMouseInfo(QMouseEvent *event);
	
	using Marker = std::tuple<int, int, uint64_t>;
	
	std::deque<Marker> _markers;
	QTimer _markerTimer;
	uint64_t _counter = 0;
};

#endif // DRAWWIDGET_H
